module ActiveRpc
  class RpcController < ActionController::Base
    if ActiveRpc.config.after_action.present?
      after_action(&ActiveRpc.config.after_action)
    end

    if ActiveRpc.config.before_action.present?
      before_action(&ActiveRpc.config.before_action)
    end

    def create
      res = nil
      begin
        res = case body
              when Array then body.map(&method(:process_item)).map(&:to_hash)
              when Hash then process_item(body).to_hash
              end
      rescue JSON::ParserError => ex
        res = ActiveRpc::Response.new do |r|
          r.error = Errors::ParseError.new(message: ex.to_s)
        end.to_hash
      end

      render status: :ok, json: res
    end

    private def process_item(item)
      req = ActiveRpc::Request.new(item.slice('id', 'method', 'params'))
      res = ActiveRpc::Response.from_request(req)
      ex = nil

      begin
        raise TypeError, 'invalid JSON-RPC request' unless req.valid?

        executor = ActiveRpc.get_executor(req.method)
        raise NoMethodError, "undefined operation `#{req.method}'" unless executor

        ex = executor.new(req.params)
        raise ArgumentError, 'invalid payload' unless ex.valid?
        res.result = ex.call
      rescue TypeError => e
        res.error = Errors::ClientError.new(message: e.to_s)
        res.error.data = req.errors
      rescue NoMethodError => e
        res.error = Errors::NoMethodError.new(message: e.to_s)
      rescue ArgumentError => e
        res.error = Errors::ArgumentError.new(message: e.to_s)
        res.error.data = ex.errors
      rescue OperationFailure => e
        res.error = e.rpc_error
      # todo: handle rpc-level errors
      rescue => e
        res.error = Errors::InternalError.new(message: e.to_s)
      end

      res
    end

    private def body
      @body ||= JSON.load(request.body.read)
    end
  end
end
