module ActiveRpc
  class OperationFailure < RuntimeError
    attr_reader :code, :message, :data

    def initialize(code:, message: nil, data: nil)
      @code = code
      @message = message
      @data = data
    end

    def rpc_error
      Errors::OperationFailure.new(code: code, message: message, data: data)
    end
  end

  class Error
    attr_accessor :message, :data
    attr_reader :code
    def initialize(code:, data: nil, message: nil)
      @message = message
      @code = code
      @data = data
    end

    def to_hash
      {
        'code' => code,
        'data' => data,
        'message' => message
      }.compact
    end
  end

  module Errors
    class ClientError < Error
      def initialize(message:)
        super(message: message, code: -32600)
      end
    end

    class NoMethodError < Error
      def initialize(message:)
        super(message: message, code: -32601)
      end
    end

    class ParseError < Error
      def initialize(message:)
        super(message: message, code: -32700)
      end
    end

    class ArgumentError < Error
      def initialize(message:)
        super(message: message, code: -32602)
      end
    end

    class InternalError < Error
      def initialize(message:)
        super(message: message, code: -32603)
      end
    end

    class ServerError < Error
      def initialize(message:)
        super(message: message, code: -32000)
      end
    end

    class OperationFailure < Error
      def initialize(code:, message:, data: nil)
        super(message: message, code: code, data: data)
      end
    end
  end
end
