require 'active_model/model'
require 'active_model/validations'

module ActiveRpc
  class Request
    include ActiveModel::Model
    include ActiveModel::Validations

    attr_accessor :id, :method, :params

    validates_each :id do |record, field, value|
      unless value.is_a?(String) || value.is_a?(Integer)
        record.errors.add(field, %(illegal type - received #{value.class}))
      end
    end

    validates_each :method do |record, field, value|
      unless value.is_a?(String)
        record.errors.add(field, %(illegal type - received #{value.class}))
      end

      unless value.present?
        record.errors.add(field, 'must not be empty')
      end
    end

    validates_each :params do |record, field, value|
      unless value.nil? || value.is_a?(Array) || value.is_a?(Hash)
        record.errors.add(field, %(invalid type - received #{value.class}))
      end
    end
  end
end
